package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.Hibernate;

import br.com.rjconsultores.auditador.annotations.AuditarClasse;
import br.com.rjconsultores.auditador.annotations.NaoAuditar;
import br.com.rjconsultores.auditador.interfaces.Auditavel;

@Entity
@AuditarClasse(nome = "CategoriaRuta", tela = "auditarClasse.CategoriaRuta")
@SequenceGenerator(name = "CATEGORIA_RUTA_SEQ", sequenceName = "CATEGORIA_RUTA_SEQ", allocationSize = 1)
@Table(name = "CATEGORIA_RUTA")
public class CategoriaRuta implements Serializable, Auditavel<CategoriaRuta> {

    private static final long serialVersionUID = 1L;
    @Id
    @Basic(optional = false)
    @GeneratedValue(strategy = GenerationType.AUTO, generator = "CATEGORIA_RUTA_SEQ")
    @Column(name = "CATEGORIARUTA_ID")
    private Integer categoriarutaId;
    @Column(name = "ACTIVO")
    private Boolean activo;
    @Column(name = "FECMODIF")
    @Temporal(TemporalType.TIMESTAMP)
    private Date fecmodif;
    @Column(name = "USUARIO_ID")
    private Integer usuarioId;
    @Column(name = "QTD_POLTRONA_AUTORIZADA")
    private Integer qtdPoltronasAutorizadas;
    @Column(name = "VALOR_DESCONTO")
    private BigDecimal valorDesconto;
    @Column(name = "PORCENTAGEM_DESCONTO")
    private BigDecimal porcentagemDesconto;
    
    @JoinColumn(name = "RUTA_ID", referencedColumnName = "RUTA_ID")
    @ManyToOne
    private Ruta ruta;
    @JoinColumn(name = "CATEGORIACTRL_ID", referencedColumnName = "CATEGORIACTRL_ID")
    @ManyToOne
    private CategoriaCtrl categoriaCtrl;
    
	@Transient
	@NaoAuditar
	private CategoriaRuta categoriaRutaClone;

    public CategoriaRuta() {
    }

    public CategoriaRuta(Integer categoriarutaId) {
        this.categoriarutaId = categoriarutaId;
    }

    public Integer getCategoriarutaId() {
		return categoriarutaId;
	}

	public void setCategoriarutaId(Integer categoriarutaId) {
		this.categoriarutaId = categoriarutaId;
	}

	public Boolean getActivo() {
        return activo;
    }

    public void setActivo(Boolean activo) {
        this.activo = activo;
    }

    public Date getFecmodif() {
        return fecmodif;
    }

    public void setFecmodif(Date fecmodif) {
        this.fecmodif = fecmodif;
    }

    public Integer getUsuarioId() {
        return usuarioId;
    }

    public void setUsuarioId(Integer usuarioId) {
        this.usuarioId = usuarioId;
    }
      

	public Integer getQtdPoltronasAutorizadas() {
		return qtdPoltronasAutorizadas;
	}

	public void setQtdPoltronasAutorizadas(Integer qtdPoltronasAutorizadas) {
		this.qtdPoltronasAutorizadas = qtdPoltronasAutorizadas;
	}

	public BigDecimal getValorDesconto() {
		return valorDesconto;
	}

	public void setValorDesconto(BigDecimal valorDesconto) {
		this.valorDesconto = valorDesconto;
	}

	public BigDecimal getPorcentagemDesconto() {
		return porcentagemDesconto;
	}

	public void setPorcentagemDesconto(BigDecimal porcentagemDesconto) {
		this.porcentagemDesconto = porcentagemDesconto;
	}

	public Ruta getRuta() {
		return ruta;
	}

	public void setRuta(Ruta ruta) {
		this.ruta = ruta;
	}

	public CategoriaCtrl getCategoriaCtrl() {
        return categoriaCtrl;
    }

    public void setCategoriaCtrl(CategoriaCtrl categoriaCtrl) {
        this.categoriaCtrl = categoriaCtrl;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final CategoriaRuta other = (CategoriaRuta) obj;
        if (this.ruta != other.ruta && (this.ruta == null || !this.ruta.equals(other.ruta))) {
            return false;
        }
        if (this.categoriaCtrl != other.categoriaCtrl && (this.categoriaCtrl == null || !this.categoriaCtrl.equals(other.categoriaCtrl))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 97 * hash + (this.ruta != null ? this.ruta.hashCode() : 0);
        hash = 97 * hash + (this.categoriaCtrl != null ? this.categoriaCtrl.hashCode() : 0);
        return hash;
    }

    @Override
    public String toString() {
        return "com.rjconsultores.ventaboletos.entidad.CategoriaOrgao[categoriaorgaoId=" + categoriarutaId + "]";
    }
    
	@Override
	public void clonar() throws CloneNotSupportedException {
		categoriaRutaClone = new CategoriaRuta();
		categoriaRutaClone = (CategoriaRuta) this.clone();
		Hibernate.initialize(categoriaRutaClone.getCategoriaCtrl());

	}

	@Override
	public CategoriaRuta getCloneObject() throws CloneNotSupportedException {
		return categoriaRutaClone;
	}

	@Override
	public String getTextoInclusaoExclusao() {
		return String.format("ID [%s]", getRuta().getNumRuta());
	}

}
