package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.ConvenioTransportadoraDAO;
import com.rjconsultores.ventaboletos.dao.TarifaConvenioTransportDAO;
import com.rjconsultores.ventaboletos.entidad.ConvenioTransportadora;
import com.rjconsultores.ventaboletos.entidad.TarifaConvenioTransport;
import com.rjconsultores.ventaboletos.exception.BusinessException;
import com.rjconsultores.ventaboletos.service.ConvenioTransportadoraService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("convenioTransportadoraService")
public class ConvenioTransportadoraServiceImpl implements ConvenioTransportadoraService {

    @Autowired
    private ConvenioTransportadoraDAO convenioTransportadoraDAO;
    
    @Autowired
    private TarifaConvenioTransportDAO tarifaConvenioDAO;

    public List<ConvenioTransportadora> obtenerTodos() {
        return convenioTransportadoraDAO.obtenerTodos();
    }

    public ConvenioTransportadora obtenerID(Long id) {
        return convenioTransportadoraDAO.obtenerID(id);
    }

    @Transactional
    public ConvenioTransportadora suscribir(ConvenioTransportadora entidad) {
    	entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecModif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);
    	
        return convenioTransportadoraDAO.suscribir(entidad);
    }
    
    @Transactional
	public ConvenioTransportadora suscribirActualizar(ConvenioTransportadora entidad) throws BusinessException  {
		
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecModif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		if (entidad.getConvenioTransportadoraId() == null){
			return convenioTransportadoraDAO.suscribir(entidad);
		}else{
			return convenioTransportadoraDAO.actualizacion(entidad);
		}
	}

    @Transactional
    public ConvenioTransportadora actualizacion(ConvenioTransportadora entidad) {
        return convenioTransportadoraDAO.actualizacion(entidad);
    }

    @Transactional
    public void borrar(ConvenioTransportadora entidad) {
    	entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecModif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.FALSE);
        
        convenioTransportadoraDAO.borrar(entidad);
    }

	@Override
	public ConvenioTransportadora buscarPelaTransportadoraId(Long transportadoraId) {
		return convenioTransportadoraDAO.buscarPelaTransportadoraId(transportadoraId) ;
	}

	@Override
	public boolean existe(ConvenioTransportadora convenio) {
		return convenioTransportadoraDAO.existe(convenio);
	}

	@Transactional
	public boolean gravarTarifasConvenio(List<TarifaConvenioTransport> lsTarifa) {
		return tarifaConvenioDAO.gravarTarifasConvenio(lsTarifa);
		
	}
}
