/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;
import java.util.Map.Entry;
import java.util.Properties;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.CustomDAO;
import com.rjconsultores.ventaboletos.entidad.Custom;
import com.rjconsultores.ventaboletos.enums.CustomTipo;
import com.rjconsultores.ventaboletos.enums.SistemaEnum;
import com.rjconsultores.ventaboletos.service.CustomService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

/**
 * 
 * @author Lucas
 */
@Service("customService")
public class CustomServiceImpl implements CustomService {

	@Autowired
	private CustomDAO customDAO;

	public List<Custom> obtenerTodos() {
		return customDAO.obtenerTodos();
	}

	public Custom obtenerID(Integer id) {
		return customDAO.obtenerID(id);
	}

	@Transactional(propagation = Propagation.SUPPORTS)
	public Custom suscribir(Custom entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado() != null ? UsuarioLogado.getUsuarioLogado().getUsuarioId() : null);
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return customDAO.suscribir(entidad);
	}

	@Transactional
	public Custom actualizacion(Custom entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return customDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(Custom entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		customDAO.actualizacion(entidad);
	}
	
	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public Custom buscar(String chave) {
		return customDAO.buscar(chave);
	}

	@Override
	@Transactional
	public void carregarArquivoProperties(SistemaEnum sistema, Properties arquivo, boolean substituirExistente) {
		if(arquivo != null) {
			for (Entry<Object, Object> config : arquivo.entrySet()) {
				Custom custom = customDAO.buscar(sistema.getValor(), config.getKey().toString());
				
				if(custom != null && !substituirExistente) {
					continue;
				}
				
				if(custom == null) {
					custom = new Custom();
					custom.setSistema(sistema.getValor());
					custom.setOrdem(0);
					custom.setChave(config.getKey().toString());
					if(sistema.equals(SistemaEnum.VENDA_SERVIDOR)) {
						custom.setTipo(CustomTipo.TIPO_SERVIDOR.getValor());
					} else if(sistema.equals(SistemaEnum.VENDA_CLIENTE)) {
						custom.setTipo(CustomTipo.TIPO_CLIENTE.getValor());
					}
				}
				
				custom.setValor(config.getValue().toString());
				suscribirOrActualizacion(custom);
			}
		}
	}
	
	@Transactional
	private Custom suscribirOrActualizacion(Custom custom) {
		if(custom.getCustomId() == null) {
			return suscribir(custom);
		} else {
			return actualizacion(custom);
		}
	}
}
